// Ryan Schmidt - rms@unknownroad.com
// Copyright (c) 2005-2010
// Distributed under the Boost Software License, Version 1.0.
// http://www.boost.org/LICENSE_1_0.txt


// HUDManager.cpp : implementation file
//

#include "stdafx.h"
#include "BlobShop.h"
#include "HUDManager.h"
#include "SystemConstants.h"
#include "HUDButtonListeners.h"
#include "SceneManager.h"

#include "rmsdebug.h"

#include <GL/gl.h>
#include <GL/glu.h>

#include <WmlExtVectorUtil.h>



class HUDPopUpToolbarCloser : public rmsui::IHUDPopUpToolbarEventHandler
{
public:
	HUDPopUpToolbarCloser( rmsui::HUDPopUpToolbar * pToolbar, float fCloseDelta ) 
		{ m_pToolbar = pToolbar; m_fCloseDelta = fCloseDelta; }
	virtual ~HUDPopUpToolbarCloser() {}

	virtual void OnHUDPopUpToolbarExpand() {
		m_openTime = rmsui::TimeEvent::Now();
	}

	bool OnClockTick( const rmsui::TimeEvent & curTime ) {
		if ( m_pToolbar->IsExpanded() && curTime.SecondsDelta(m_openTime) > m_fCloseDelta ) {
			m_pToolbar->ForceContract();
			return true;
		} else
			return false;
	}

protected:
	rmsui::HUDPopUpToolbar * m_pToolbar;
	float m_fCloseDelta;
	rmsui::TimeEvent m_openTime;
};




// CHUDManager

IMPLEMENT_DYNAMIC(CHUDManager, CWnd)
CHUDManager::CHUDManager()
	: m_pView(NULL)
{
	// add to event listener
	//rmsui::CInputEventStream::Get()->AddListener(this);

	static const float fButtonSize = 0.125f;
//	static const float fButtonSize = 0.25f;

	rmsui::HUDFileIcon * pIcon = new rmsui::HUDFileIcon(  CSystemConstants::GetIconPath() + CString("zoom.png") );
	m_pZoomButton = new rmsui::HUDButton("ZoomButton", pIcon, true);
	m_pZoomButton->Bounds() = Wml::AxisAlignedBox2f(0.0f, fButtonSize, 0.0f, fButtonSize);

	rmsui::HUDFileIcon * pIcon2 = new rmsui::HUDFileIcon( CSystemConstants::GetIconPath() + CString("pan.png") );
	m_pPanButton = new rmsui::HUDButton("PanButton", pIcon2, true);
	m_pPanButton->Bounds() = Wml::AxisAlignedBox2f(0.0f, fButtonSize, 0.0f, fButtonSize);

	rmsui::HUDFileIcon * pIcon3 = new rmsui::HUDFileIcon( CSystemConstants::GetIconPath() + CString("orbit.png") );
	m_pOrbitButton = new rmsui::HUDButton("OrbitButton", pIcon3, true);
	m_pOrbitButton->Bounds() = Wml::AxisAlignedBox2f(0.0f, fButtonSize, 0.0f, fButtonSize);

	// insert toolbar
	m_pViewToolbar = new rmsui::HUDToolbar( "ViewToolbar", true );
	m_pViewToolbar->AppendItem( m_pZoomButton );
	m_pViewToolbar->AppendItem( m_pPanButton );
	m_pViewToolbar->AppendItem( m_pOrbitButton );


	// create view buttons and popup
	m_pViewDirectionPopUp = new rmsui::HUDPopUpToolbar("ViewDirectionPopUp", rmsui::HUDPopUpToolbar::Vertical);
	const char *ViewButtonNames[] = { "ViewFrontButton", "ViewBackButton", 
									  "ViewTopButton", "ViewBottomButton",
									  "ViewLeftButton", "ViewRightButton" };
	const char *ViewButtonFiles[] = { "viewFront.png", "viewBack.png", 
									  "viewTop.png", "viewBottom.png",
									  "viewLeft.png", "viewRight.png" };
	for ( int i = 0; i < 6; ++i ) {
		rmsui::HUDFileIcon * pFileIcon = new rmsui::HUDFileIcon( CSystemConstants::GetIconPath() + CString(ViewButtonFiles[i]) );
		m_pViewDirectionButtons[i] = new rmsui::HUDButton( "ViewFrontButton", pFileIcon, false, true );
		m_pViewDirectionButtons[i]->Bounds() = Wml::AxisAlignedBox2f(0.0f, fButtonSize, 0.0f, fButtonSize);
		m_pViewDirectionPopUp->AppendItem( m_pViewDirectionButtons[i] );
		m_pViewDirectionHandlers[i] = NULL;
	}
	m_pViewToolbar->AppendItem( m_pViewDirectionPopUp );


	m_pViewToolbar->Translate(-0.2f, 1.0f - fButtonSize, false);

	m_vHUDItems.insert( m_pViewToolbar );

	m_pZoomButtonHandler = NULL;
	m_pPanButtonHandler = NULL;
	m_pOrbitButtonHandler = NULL;
	m_pViewDirectionPopUpCloser = NULL;

	m_pToolbarMoveHandler = NULL;

	m_pCaptureItem = NULL;
}

CHUDManager::~CHUDManager()
{
}


void CHUDManager::AddHUDButton( rmsui::HUDButton * pButton )
{
	m_vHUDItems.insert(pButton);
}

void CHUDManager::AddHUDToolbar( rmsui::HUDToolbar * pToolbar )
{
	ASSERT( m_pToolbarMoveHandler );

	m_vHUDItems.insert(pToolbar);

	// register listener that refreshes screen on toolbar move event
	pToolbar->AddListener( m_pToolbarMoveHandler );
}



void CHUDManager::SetViewController( CViewController * pView )
{
	m_pView = pView;
	ASSERT(m_pView);
	m_pView->RegisterListener(this);

	// add handlers
	if ( m_pZoomButtonHandler != NULL )
		delete m_pZoomButtonHandler;
	m_pZoomButtonHandler = new HUDZoomButtonListener( m_pView );
	m_pZoomButton->AddListener(m_pZoomButtonHandler);
	
	if ( m_pPanButtonHandler != NULL )
		delete m_pPanButtonHandler;
	m_pPanButtonHandler = new HUDPanButtonListener( m_pView );
	m_pPanButton->AddListener( m_pPanButtonHandler );

	if ( m_pOrbitButtonHandler != NULL )
		delete m_pOrbitButtonHandler;
	m_pOrbitButtonHandler = new HUDOrbitButtonListener( m_pView );
	m_pOrbitButton->AddListener( m_pOrbitButtonHandler );

	for ( int i = 0; i < 6; ++i ) {
		if ( m_pViewDirectionHandlers[i] != NULL )
			delete m_pViewDirectionHandlers[i];
		m_pViewDirectionHandlers[i] = 
			new HUDStoredViewButtonListener( m_pView, (HUDStoredViewButtonListener::ViewDirection)i );
		m_pViewDirectionButtons[i]->AddListener( m_pViewDirectionHandlers[i] );
	}

	// create close handler for popup toolbar
//	m_pViewDirectionPopUpCloser = new HUDPopUpToolbarCloser(m_pViewDirectionPopUp, 4.0f);
//	m_pViewDirectionPopUp->AddListener( m_pViewDirectionPopUpCloser );

	// create default handler for toolbar move events
	if ( m_pToolbarMoveHandler != NULL )
		delete m_pToolbarMoveHandler;
	m_pToolbarMoveHandler = new HUDViewToolbarListener( m_pView );

	m_pViewToolbar->AddListener( m_pToolbarMoveHandler );

	m_pViewDirectionPopUp->AddListener( this );
}

void CHUDManager::ToHUDParams( Wml::Vector2f & vPoint )
{
	vPoint[0] = vPoint[0] * 2.0f * m_fToHUDSpaceParams[2] - m_fToHUDSpaceParams[0];
	vPoint[1] = vPoint[1] * 2.0f * m_fToHUDSpaceParams[3] - m_fToHUDSpaceParams[1];
}



void CHUDManager::Render()
{
	glMatrixMode(GL_PROJECTION);
	glPushMatrix();
	glLoadIdentity();
	
	gluOrtho2D( -m_fToHUDSpaceParams[0], m_fToHUDSpaceParams[0], -m_fToHUDSpaceParams[1], m_fToHUDSpaceParams[1] );

	glMatrixMode(GL_MODELVIEW);
	glPushMatrix();
	glLoadIdentity();
	
	glPushAttrib(GL_ENABLE_BIT);
	glDisable(GL_LIGHTING);
	glDisable(GL_DEPTH_TEST);

	glEnable(GL_BLEND);
	glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);

	std::set< rmsui::HUDItem * >::iterator cur(m_vHUDItems.begin()), end(m_vHUDItems.end());
	while ( cur != end ) {
		rmsui::HUDItem * pItem = (*cur++);
		if ( pItem->Visible() )
			pItem->Render();
	}

	glPopAttrib();

	glMatrixMode(GL_MODELVIEW);
	glPopMatrix();

	glMatrixMode(GL_PROJECTION);
	glPopMatrix();

	glMatrixMode(GL_MODELVIEW);
}




BEGIN_MESSAGE_MAP(CHUDManager, CWnd)
END_MESSAGE_MAP()



// CHUDManager message handlers

void CHUDManager::OnViewChanged()
{
	float fAspect = (float)m_pView->GetViewport()[2] / (float)m_pView->GetViewport()[3];

	gluOrtho2D( -fAspect, fAspect, -1.0f, 1.0f );
	m_fToHUDSpaceParams[0] = fAspect;
	m_fToHUDSpaceParams[1] = 1.0f;
	m_fToHUDSpaceParams[2] = fAspect / (float)m_pView->GetViewport()[2];
	m_fToHUDSpaceParams[3] = 1.0f / (float)m_pView->GetViewport()[3];

	UpdateDockedItems();
}

bool CHUDManager::OnTimeEvent( const rmsui::TimeEvent & event )
{
	//if ( m_pViewDirectionPopUpCloser ) {
	//	if ( ( (HUDPopUpToolbarCloser *)m_pViewDirectionPopUpCloser)->OnClockTick( event ) ) {
	//		return true;
	//	}
	//}
	return false;
}

void CHUDManager::OnHUDPopUpToolbarExpand()
{
	CSceneManager::Get()->ForceReDraw();
}

void CHUDManager::OnHUDPopUpToolbarContract()
{
	CSceneManager::Get()->ForceReDraw();
}


rmsui::IInputEventListener::EventState CHUDManager::ProcessInputEvent( const rmsui::InputEventData & event )
{
	if ( m_pView == NULL )
		return IInputEventListener::Ignored;

	// convert point to proper coords
	Wml::Vector2f vPoint( (float)event.nMouseX, (float)event.nMouseY );
	ToHUDParams(vPoint);

	if ( event.LeftButton.bDown == true ) {

		// if left button was not down...
		if ( event.LeftButton.bWasDown == false ) {

			// check if any HUD items captured the input
			std::set< rmsui::HUDItem * >::iterator cur(m_vHUDItems.begin()), end(m_vHUDItems.end());
			while ( cur != end ) {
				rmsui::HUDItem * pCurItem = (*cur++);
				if ( pCurItem->Visible() == false )
					continue;
				
				rmsui::HUDItem * pCaptureItem = pCurItem->BeginConsume(vPoint);

				if ( pCaptureItem != NULL ) {
					m_pCaptureItem = pCaptureItem;
					return IInputEventListener::StartCapture;
				}
			}

		// otherwise, if the input is currently captured, forward to capture target
		} else if ( m_pCaptureItem != NULL ) {

			// capture target can force termination of capture by returning NULL
			rmsui::HUDItem * pCaptureItem = m_pCaptureItem->Consume( vPoint );
			if ( pCaptureItem == NULL ) {
				m_pCaptureItem = NULL;
				return IInputEventListener::EndCapture;
			} else
				return IInputEventListener::Consumed;

		} 

	} else if ( event.LeftButton.bWasDown == true ) {

		// if button state changed and we were in capture mode, stop capturing
		if ( m_pCaptureItem != NULL ) {
			m_pCaptureItem->EndConsume();
			m_pCaptureItem = NULL;
			return IInputEventListener::EndCapture;
		}

	} 

	return IInputEventListener::Ignored;		// not consumed...
}



bool CHUDManager::OnLineStroke( rmsui::GestureLineStroke * pLineStroke )
{
	return CheckLineStroke( pLineStroke );
}

bool CHUDManager::OnVStroke( rmsui::GestureVStroke * pVStroke )
{
	return CheckVStroke( pVStroke );
}

bool CHUDManager::OnPointStroke( rmsui::GesturePointStroke * pPointStroke )
{
	rmsui::GestureLineStroke tmp( pPointStroke->Point(), pPointStroke->Point() );
	return CheckLineStroke( &tmp );
}



bool CHUDManager::CheckLineStroke( const rmsui::GestureLineStroke * pLineStroke )
{
	// convert coords to local
	Wml::Vector2f v1( pLineStroke->EndPoint1() );
	Wml::Vector2f v2( pLineStroke->EndPoint2() );
	ToHUDParams(v1);
	ToHUDParams(v2);

	rmsui::GestureLineStroke viewLocalStroke(v1, v2);

	// check for hits
	std::set< rmsui::HUDItem * >::iterator cur(m_vHUDItems.begin()), end(m_vHUDItems.end());
	while ( cur != end ) {
		rmsui::HUDItem * pCurItem = (*cur++);
		if ( ! pCurItem->Visible() )
			continue;

		rmsui::HUDItem * pStrokeItem = pCurItem->TestLineStroke(viewLocalStroke);
		if ( pStrokeItem != NULL )
			return true;
	}	

	return false;
}


/*
 * V stroke is used to summon view tools. 
 */
bool CHUDManager::CheckVStroke( const rmsui::GestureVStroke * pVStroke )
{
	// convert coords to local
	Wml::Vector2f v1( pVStroke->Start() );
	Wml::Vector2f v2( pVStroke->End() );
	ToHUDParams(v1);
	ToHUDParams(v2);

	// move view toolbar
	m_pViewToolbar->Translate( v1.X(), v1.Y(), false );

	m_pViewToolbar->Visible() = true;

	return true;
}


void CHUDManager::UpdateDockedItems()
{
	std::set< rmsui::HUDItem * >::iterator cur(m_vHUDItems.begin()), end(m_vHUDItems.end());
	while ( cur != end ) {
		rmsui::HUDItem * pCurItem = (*cur++);
		if ( pCurItem->GetDockMode() == rmsui::HUDItem::NoDocking )
			continue;

		// get bounds of item
		Wml::AxisAlignedBox2f itemBounds(pCurItem->Bounds());

		// get viewport
		const unsigned int * pViewport = m_pView->GetViewport();

		switch ( pCurItem->GetDockMode() ) {
			case rmsui::HUDItem::DockLeft:
				{
				Wml::Vector2f vLeft(1, 0);
				ToHUDParams(vLeft);
				
				pCurItem->Translate( vLeft.X(), itemBounds.GetYMin(), false );
				}
				break;

			case rmsui::HUDItem::DockRight:
				{
				Wml::Vector2f vRight((float)pViewport[2], 0);
				ToHUDParams(vRight);
				
				pCurItem->Translate( vRight.X() - Wml::GetDimension(itemBounds,0), 
					itemBounds.GetYMin(), false );
				}
				break;

			case rmsui::HUDItem::DockBottom:
				{
				Wml::Vector2f vBottom(0, 1);
				ToHUDParams( vBottom );

				pCurItem->Translate( itemBounds.GetXMin(), vBottom.Y(), false );
				}
				break;
		}

	}	
}


