// Ryan Schmidt - rms@unknownroad.com
// Copyright (c) 2005-2010
// Distributed under the Boost Software License, Version 1.0.
// http://www.boost.org/LICENSE_1_0.txt


#pragma once

#include "SceneObject.h"
#include "HUDToolbar.h"
#include "ChangeListener.h"
#include "RenderingManager.h"

#include <vector>
#include <map>

// predecl
class DraggableFloatParamButtonHandler;
class PolygonizerResolutionButtonHandler;

class ParameterEditor
	: public IChangeListener
{
public:
	static ParameterEditor * Get();

	void SetRenderManager( CRenderingManager * pManager );


	/*
	 * IChangeListener events
	 */
	virtual void OnTreeSelectionChanged( const std::vector<rmsui::SceneObject *> & newSelection );
	virtual void OnMousePickSelectionChanged( const std::vector<rmsui::SceneObject *> & newSelection );

	void SetSelection( const std::vector< rmsui::SceneObject * > & vSceneObjects );

	void SetExtraParamObject(rmsui::SceneObject * pObject);



	rmsui::HUDToolbar * GetSelectionParamToolbar() { return m_pSelectionParamToolbar; }
	rmsui::HUDToolbar * GetGlobalParamToolbar() { return m_pGlobalParamToolbar; }

protected:

	CRenderingManager * m_pRenderManager;

	/*
	 * Global Parameters
	 */
	
	rmsui::HUDToolbar * m_pGlobalParamToolbar;

	rmsui::HUDButton * m_pMeshResolutionButton;
	PolygonizerResolutionButtonHandler * m_pMeshResolutionButtonHandler;

	void InitializeGlobalParamToolbar();


	/*
	 * Current Selection Params
	 */

	rmsui::HUDToolbar * m_pSelectionParamToolbar;

	void UpdateSelectionParamToolbar( rmsui::SceneObject * pObject );

	rmsui::SceneObject * m_pExtraParamObject;


	struct ButtonInfo {
		rmsui::HUDButton * pButton;
		DraggableFloatParamButtonHandler * pHandler;
		ButtonInfo()
			{ pButton = NULL; pHandler = NULL; }
		ButtonInfo( rmsui::HUDButton * btn, DraggableFloatParamButtonHandler * handler ) 
			{ pButton = btn; pHandler = handler; }
	};


	typedef std::map< std::string, ButtonInfo > ButtonNameMap;
	ButtonNameMap m_buttonMap;
	void InitializeSelectionParamButtons();
	ButtonInfo FindButtonByName( const char * pName );

	// parameter buttons
	rmsui::HUDButton * m_pWidthButton;
	rmsui::HUDButton * m_pBlendPowerButton;
	rmsui::HUDButton * m_pTaperButton;
	rmsui::HUDButton * m_pRadiusButton;
	rmsui::HUDButton * m_pSharpnessButton;

	DraggableFloatParamButtonHandler * m_pWidthButtonHandler;
	DraggableFloatParamButtonHandler * m_pBlendPowerButtonHandler;
	DraggableFloatParamButtonHandler * m_pTaperButtonHandler;
	DraggableFloatParamButtonHandler * m_pRadiusButtonHandler;
	DraggableFloatParamButtonHandler * m_pSharpnessButtonHandler;


	void DoParameterNameRewritingHacks(rmsui::SceneObject * pObject, CString & paramName );

	// singleton stuff
	static ParameterEditor * s_pSingleton;
	ParameterEditor();
	virtual ~ParameterEditor() {}
};

