// Ryan Schmidt - rms@unknownroad.com
// Copyright (c) 2005-2010
// Distributed under the Boost Software License, Version 1.0.
// http://www.boost.org/LICENSE_1_0.txt


// ParametersDialogBar.cpp : implementation file
//

#include "stdafx.h"
#include "BlobShop.h"
#include "ParametersDialogBar.h"
#include ".\parametersdialogbar.h"
#include <SceneObject.h>

#include "MainFrm.h"
#include "ChildFrm.h"
#include "Standard3DView.h"


CParametersDialogBar * CParametersDialogBar::m_spSingleton = NULL; 


// CParametersDialogBar

IMPLEMENT_DYNAMIC(CParametersDialogBar, CDialogBar)
CParametersDialogBar::CParametersDialogBar()
{
	m_bColumnsInitialized = false;
}

CParametersDialogBar::~CParametersDialogBar()
{
}

CParametersDialogBar * CParametersDialogBar::Get()
{
	if (m_spSingleton == NULL) {
		m_spSingleton = new CParametersDialogBar();
		ASSERT(m_spSingleton);
	}
	return m_spSingleton;
}


void CParametersDialogBar::OnTreeSelectionChanged( const std::vector<rmsui::SceneObject *> & newSelection )
{
	if (newSelection.size() == 1) {
		SetParameters( newSelection[0]->GetParameterSet() );
	} else {
		ClearParameters();
	}
}

void CParametersDialogBar::OnMousePickSelectionChanged( const std::vector<rmsui::SceneObject *> & newSelection )
{
	if (newSelection.size() == 1) {
		SetParameters( newSelection[0]->GetParameterSet() );
	} else {
		ClearParameters();
	}
}


void CParametersDialogBar::OnSelectionParametersChanged( )
{
	SynchronizeCurrentParameters();
}

void CParametersDialogBar::InitializeColumns()
{
	if (m_bColumnsInitialized == true)
		return;

	CListCtrl * pCtrl = (CListCtrl *)GetDlgItem(IDC_PARAMSDIALOGBAR_PARAMLIST);
	ASSERT(pCtrl);

	ListView_SetExtendedListViewStyle( pCtrl->GetSafeHwnd(), LVS_EX_GRIDLINES | LVS_EX_ONECLICKACTIVATE   );

	CRect rect;
	pCtrl->GetClientRect(rect);
	int nFirstColWidth = (int)( 0.5f * (float)rect.Width() );

	pCtrl->InsertColumn(0, "Value", LVCFMT_LEFT, nFirstColWidth, -1);
	pCtrl->InsertColumn(1, "Name", LVCFMT_LEFT, rect.Width() - nFirstColWidth, 1 );

	int order[2] = {1,0};
	pCtrl->SetColumnOrderArray( 2, order );

	m_bColumnsInitialized = true;
}


void CParametersDialogBar::SetParameters( const rmsui::ParameterList & paramList )
{
	InitializeColumns();

	CListCtrl * pCtrl = (CListCtrl *)GetDlgItem(IDC_PARAMSDIALOGBAR_PARAMLIST);
	ASSERT(pCtrl);
	pCtrl->DeleteAllItems();

	const std::vector<rmsui::IParameter *> & params = paramList.GetParameters();
	size_t nSize = params.size();

	CString str;
	for (unsigned int i = 0; i < nSize; ++i) {

		// is this necessary here ??
		params[i]->SynchronizeParameter();

		LVITEM valueItem;
		valueItem.iItem = i;
		valueItem.iSubItem = 0;
		params[i]->GetValue(str);
		valueItem.pszText = (LPSTR)(LPCSTR)str;
		valueItem.cchTextMax = str.GetLength();
		valueItem.mask = LVIF_TEXT | LVIF_PARAM;
		valueItem.lParam = (LPARAM)params[i];

		pCtrl->InsertItem(&valueItem);

		LVITEM nameItem;
		nameItem.iItem = i;
		nameItem.iSubItem = 1;
		str = params[i]->GetName();
		nameItem.pszText = (LPSTR)(LPCSTR)str;
		nameItem.cchTextMax = str.GetLength();
		nameItem.mask = LVIF_TEXT;

		pCtrl->SetItem(&nameItem);
	}

}


void CParametersDialogBar::SynchronizeCurrentParameters()
{
	CListCtrl * pCtrl = (CListCtrl *)GetDlgItem(IDC_PARAMSDIALOGBAR_PARAMLIST);
	ASSERT(pCtrl);

	int nCount = pCtrl->GetItemCount();
	for (int i = 0; i < nCount; ++i) {

		rmsui::IParameter * pParam = (rmsui::IParameter *)pCtrl->GetItemData(i);
		pParam->SynchronizeParameter();

		LVITEM valueItem;
		valueItem.iItem = i;
		valueItem.iSubItem = 0;
		CString str;
		pParam->GetValue(str);
		valueItem.pszText = (LPSTR)(LPCSTR)str;
		valueItem.cchTextMax = str.GetLength();
		valueItem.mask = LVIF_TEXT | LVIF_PARAM;
		valueItem.lParam = (LPARAM)pParam;

		pCtrl->SetItem(&valueItem);
	}
}

void CParametersDialogBar::ClearParameters()
{
	CListCtrl * pCtrl = (CListCtrl *)GetDlgItem(IDC_PARAMSDIALOGBAR_PARAMLIST);
	ASSERT(pCtrl);

	pCtrl->DeleteAllItems();
}


BEGIN_MESSAGE_MAP(CParametersDialogBar, CDialogBar)
	ON_NOTIFY(LVN_BEGINLABELEDIT, IDC_PARAMSDIALOGBAR_PARAMLIST, OnLvnBeginlabeleditParamsdialogbarParamlist)
	ON_NOTIFY(LVN_ITEMACTIVATE, IDC_PARAMSDIALOGBAR_PARAMLIST, OnLvnItemActivateParamsdialogbarParamlist)
	ON_NOTIFY(LVN_ENDLABELEDIT, IDC_PARAMSDIALOGBAR_PARAMLIST, OnLvnEndlabeleditParamsdialogbarParamlist)
	ON_WM_LBUTTONUP()
END_MESSAGE_MAP()



// CParametersDialogBar message handlers


void CParametersDialogBar::OnLvnBeginlabeleditParamsdialogbarParamlist(NMHDR *pNMHDR, LRESULT *pResult)
{
	NMLVDISPINFO *pDispInfo = reinterpret_cast<NMLVDISPINFO*>(pNMHDR);
	// TODO: Add your control notification handler code here
	*pResult = 0;
}


void CParametersDialogBar::OnLvnEndlabeleditParamsdialogbarParamlist(NMHDR *pNMHDR, LRESULT *pResult)
{
	NMLVDISPINFO *pDispInfo = reinterpret_cast<NMLVDISPINFO*>(pNMHDR);
	
	CListCtrl * pCtrl = (CListCtrl *)GetDlgItem(IDC_PARAMSDIALOGBAR_PARAMLIST);
	ASSERT(pCtrl);

	if ( pDispInfo->item.mask & LVIF_TEXT) {

		rmsui::IParameter * pParam = (rmsui::IParameter *)pDispInfo->item.lParam;

		pParam->SetValue( pDispInfo->item.pszText );

		CMainFrame * pMainFrame = (CMainFrame *)AfxGetMainWnd();
		CChildFrame * pFrame = (CChildFrame *)pMainFrame->GetActiveFrame();
		CStandard3DView * pView = dynamic_cast<CStandard3DView *>(pFrame->GetActiveView());
		if (pView == NULL) {
			MessageBox("ack! current view is not main view... ignored option change!");
			*pResult = 0;
			return;
		}
		//CStandard3DView * pView = (CStandard3DView *)pFrame->GetActiveView();
		pView->GetSceneManager().NotifyToolActionOcurred();

		LVITEM item2;
		item2.iItem = pDispInfo->item.iItem;
		item2.iSubItem = 0;
		CString str;
		pParam->GetValue(str);
		item2.pszText = (LPSTR)(LPCSTR)str;
		item2.cchTextMax = str.GetLength();
		item2.mask = LVIF_TEXT;	

		pCtrl->SetItem(&item2);
	}

	*pResult = 0;
}


void CParametersDialogBar::OnLvnItemActivateParamsdialogbarParamlist(NMHDR *pNMHDR, LRESULT *pResult)
{
	LPNMITEMACTIVATE pItem = reinterpret_cast<LPNMITEMACTIVATE>(pNMHDR);
	
	CListCtrl * pCtrl = (CListCtrl *)GetDlgItem(IDC_PARAMSDIALOGBAR_PARAMLIST);
	ASSERT(pCtrl);

	if (pItem->iSubItem == 0)
		ListView_EditLabel( pCtrl->GetSafeHwnd(), pItem->iItem );

	*pResult = 0;
}





void CParametersDialogBar::OnLButtonUp(UINT nFlags, CPoint point)
{
	// TODO: Add your message handler code here and/or call default

	__super::OnLButtonUp(nFlags, point);
}
