// Ryan Schmidt - rms@unknownroad.com
// Copyright (c) 2005-2010
// Distributed under the Boost Software License, Version 1.0.
// http://www.boost.org/LICENSE_1_0.txt



#pragma once

#include "ChangeListener.h"
#include <ToolManager.h>
#include "SceneObjectManager.h"

#include <vector>
#include <set>

#include <GestureEngine.h>


// CSceneManager command target


typedef CWnd * CurrentSceneManagerKey;

class CSceneManager 
	: public CWnd,
	  public rmsui::IToolActionEventListener,
	  public rmsui::IGestureEventListener
{
	DECLARE_DYNAMIC(CSceneManager)

private:
	CSceneManager();
	virtual ~CSceneManager();

	// list of streams
	static std::map< CurrentSceneManagerKey, CSceneManager * > s_streams;
	static CurrentSceneManagerKey s_CurrentKey;

public:
	static void SetCurrentSceneManager( CurrentSceneManagerKey key );
	static CSceneManager * Get();

	void SetTool3DManager( rmsui::Tool3DManager * pTool3DManager );

	void RegisterChangeListener( IChangeListener * pListener );

	inline rmsui::SceneObjectManager * GetSceneObjectManager()
		{ return m_pObjectManager; }

	void ForceToolManagerUpdate();

	void ForceReDraw();


	/*
	 * rmsui::IToolActionEventListener handlers
	 */
	virtual void OnToolMoveEvent();

	/*
	 * things related to tool manager
	 */
	void NotifyToolActionOcurred();
	void NotifyMousePickActionOccurred();

	/*
	 * direct implicit picking support
	 */
	bool TryPickSurface( const Wml::Ray3f & ray );
	bool MoveSelectionToParent();

	/*
	 * message pass-through that sketch manager currently uses
	 */
	void NotifyParametersChangedPassThrough();


	/*
	 * things related to CBlobTreeEditorDialogBar...
	 */

	const std::vector<rmsui::SceneObject *> & GetCurrentSelectionSet()
		{ return m_curSelection; }
	void ClearBlobTreeEditorTreeSelectionSet();


	void SetTemporaryCuttingPlane( const Wml::Frame3f & planeFrame );
	bool IsTemporaryCuttingPlaneEnabled();
	void ClearTemporaryCuttingPlane();

	// misc util functions
	void FlipSelection( const Wml::Matrix3f & matFlip );
	Wml::SerializerRecord * SerializeSelection( Wml::Serializer * pSerializer );
	bool HasSelection() { return m_curSelection.size() > 0; }


protected:
	rmsui::Tool3DManager * m_pTool3DManager;		// this is external...

	rmsui::SceneObjectManager * m_pObjectManager;


	// get rid of this
	std::vector<rmsui::SceneObject *> m_curSelection;



	std::set<IChangeListener *> m_setListeners;
	void NotifySceneChanged();
	void NotifyTreeSelectionChanged();
	void NotifyMousePickSelectionChanged();
	void NotifySelectionParametersChanged();
	void NotifyCurrentBlobTreeChanged();
	void NotifyCuttingPlaneEnabled( const Wml::Frame3f & vPlaneFrame );
	void NotifyCuttingPlaneDisabled();
	void NotifyRenderParametersChanged();


	// Creation functions for loader
public:



	DECLARE_MESSAGE_MAP()
public:
	afx_msg void OnRemoveSelection();
	afx_msg void OnAddprimPoint();

	afx_msg void OnEditClearselection();
};


