// Ryan Schmidt - rms@unknownroad.com
// Copyright (c) 2004-2010
// Distributed under the Boost Software License, Version 1.0.
// http://www.boost.org/LICENSE_1_0.txt



#include "stdafx.h"
#include "SketchFramework.h"
#include "MainFrm.h"

#include "ChildFrm.h"
#include "SketchFrameworkDoc.h"
#include "Standard3DView.h"
#include "2DFieldEditor.h"

#include "FieldImageFrm.h"
#include "RayTracedFrm.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#endif


// CSketchFrameworkApp

BEGIN_MESSAGE_MAP(CSketchFrameworkApp, CWinApp)
	ON_COMMAND(ID_APP_ABOUT, OnAppAbout)
	// Standard file based document commands
	ON_COMMAND(ID_FILE_OPEN, CWinApp::OnFileOpen)
	// Standard print setup command
	ON_COMMAND(ID_FILE_PRINT_SETUP, CWinApp::OnFilePrintSetup)
END_MESSAGE_MAP()


// CSketchFrameworkApp construction

CSketchFrameworkApp::CSketchFrameworkApp()
{
	// TODO: add construction code here,
	// Place all significant initialization in InitInstance
	m_hFieldImageFrmMenu = NULL;
	m_hFieldImageFrmAccel = NULL;
	m_hBlobTreeEditorFrmMenu = NULL;
	m_hBlobTreeEditorFrmAccel = NULL;

}


// The one and only CSketchFrameworkApp object

CSketchFrameworkApp theApp;

// CSketchFrameworkApp initialization

BOOL CSketchFrameworkApp::InitInstance()
{
	// InitCommonControls() is required on Windows XP if an application
	// manifest specifies use of ComCtl32.dll version 6 or later to enable
	// visual styles.  Otherwise, any window creation will fail.
	InitCommonControls();

	CWinApp::InitInstance();

	// Standard initialization
	// If you are not using these features and wish to reduce the size
	// of your final executable, you should remove from the following
	// the specific initialization routines you do not need
	// Change the registry key under which our settings are stored
	// TODO: You should modify this string to be something appropriate
	// such as the name of your company or organization
	SetRegistryKey(_T("Local AppWizard-Generated Applications"));
	LoadStdProfileSettings(4);  // Load standard INI file options (including MRU)
	// Register the application's document templates.  Document templates
	//  serve as the connection between documents, frame windows and views


	// register window classes
	g_C2DFieldEditorClassName = AfxRegisterWndClass(
		CS_OWNDC | CS_VREDRAW | CS_HREDRAW, 
		::LoadCursor(NULL, IDC_ARROW), 
		(HBRUSH) ::GetStockObject(BLACK_BRUSH), 
		::LoadIcon(NULL, IDI_APPLICATION) );


	CMultiDocTemplate* pDocTemplate;
	pDocTemplate = new CMultiDocTemplate(IDR_SketchFrameworkTYPE,
		RUNTIME_CLASS(CSketchFrameworkDoc),
		RUNTIME_CLASS(CChildFrame), // custom MDI child frame
		RUNTIME_CLASS(CStandard3DView));
	AddDocTemplate(pDocTemplate);


	// load other resources
	HINSTANCE hInst = AfxGetResourceHandle();
	m_hFieldImageFrmMenu  = ::LoadMenu(hInst, MAKEINTRESOURCE(IDR_FieldImageTYPE));
	m_hFieldImageFrmAccel = ::LoadAccelerators(hInst, MAKEINTRESOURCE(IDR_FieldImageTYPE));


	// create main MDI Frame window
	CMainFrame* pMainFrame = new CMainFrame;
	if (!pMainFrame->LoadFrame(IDR_MAINFRAME))
		return FALSE;
	m_pMainWnd = pMainFrame;
	// call DragAcceptFiles only if there's a suffix
	//  In an MDI app, this should occur immediately after setting m_pMainWnd
	// Parse command line for standard shell commands, DDE, file open
	CCommandLineInfo cmdInfo;
	ParseCommandLine(cmdInfo);
	// Dispatch commands specified on the command line.  Will return FALSE if
	// app was launched with /RegServer, /Register, /Unregserver or /Unregister.
	if (!ProcessShellCommand(cmdInfo))
		return FALSE;
	// The main window has been initialized, so show and update it
	pMainFrame->ShowWindow(m_nCmdShow);
	pMainFrame->UpdateWindow();
	return TRUE;
}



int CSketchFrameworkApp::ExitInstance()
{
	if (m_hFieldImageFrmMenu != NULL)
		FreeResource(m_hFieldImageFrmMenu);
	if (m_hFieldImageFrmAccel != NULL)
		FreeResource(m_hFieldImageFrmAccel);

	return CWinApp::ExitInstance();
}


// CAboutDlg dialog used for App About

class CAboutDlg : public CDialog
{
public:
	CAboutDlg();

// Dialog Data
	enum { IDD = IDD_ABOUTBOX };

protected:
	virtual void DoDataExchange(CDataExchange* pDX);    // DDX/DDV support

// Implementation
protected:
	DECLARE_MESSAGE_MAP()
public:
	afx_msg void OnUpdateFieldimageStretchtofit(CCmdUI *pCmdUI);
	afx_msg void OnFieldimageStretchtofit();
};

CAboutDlg::CAboutDlg() : CDialog(CAboutDlg::IDD)
{
}

void CAboutDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
}

BEGIN_MESSAGE_MAP(CAboutDlg, CDialog)
END_MESSAGE_MAP()

// App command to run the dialog
void CSketchFrameworkApp::OnAppAbout()
{
	CAboutDlg aboutDlg;
	aboutDlg.DoModal();
}


// CSketchFrameworkApp message handlers

CMDIChildWnd * CSketchFrameworkApp::CreateNewMDIWindow(MDIWindowType eType)
{
	HINSTANCE hInst = AfxGetResourceHandle();

	CMainFrame* pFrame = STATIC_DOWNCAST(CMainFrame, m_pMainWnd);

	CMDIChildWnd * pNewChild;

	switch(eType) {
		case FieldImageWindow:
			pNewChild = pFrame->CreateNewChild( RUNTIME_CLASS(CFieldImageFrm), 
				IDR_FieldImageTYPE, m_hFieldImageFrmMenu, m_hFieldImageFrmAccel);
			break;

		case RayTracedWindow:
			pNewChild = pFrame->CreateNewChild( RUNTIME_CLASS(CRayTracedFrm),
				IDR_RayTracedTYPE, NULL, NULL );
			break;

		default:
			ASSERT(0);
			return NULL;
	}
	ASSERT(pNewChild);
	return pNewChild;
}



