// Ryan Schmidt - rms@unknownroad.com
// Copyright (c) 2005-2010
// Distributed under the Boost Software License, Version 1.0.
// http://www.boost.org/LICENSE_1_0.txt


#include "StdAfx.h"
#include ".\viewcontroller.h"

#include <WmlExtVectorUtil.h>

CViewController::CViewController(Wml::ExtendedWmlCamera * pCamera)
	: m_pCamera(pCamera)
{
	ASSERT(pCamera);
}

CViewController::~CViewController(void)
{
}

void CViewController::ForceRepaint()
{
	NotifyViewChanged();
}

void CViewController::UpdateCamera()
{
	m_pCamera->Update();
	NotifyViewChanged();
}

void CViewController::DollyZoom( float dx, float dy )
{
	m_pCamera->DollyZoom(dx + dy);
	NotifyViewChanged();
}

void CViewController::Pan( float dx, float dy )
{
	m_pCamera->PanLateral(dx);
	m_pCamera->PanVertical(dy);
	NotifyViewChanged();
}

void CViewController::Orbit( float dx, float dy )
{
	m_pCamera->OrbitLateral(-dx);
	m_pCamera->OrbitVertical(-dy);
	NotifyViewChanged();
}

void CViewController::ViewLeft()
{
	float fViewDist = ( m_pCamera->GetTarget() - m_pCamera->GetLocation() ).Length();
	m_pCamera->SetTargetFrame(
		m_pCamera->GetTarget() + -fViewDist * Wml::Vector3f::UNIT_X,
		-Wml::Vector3f::UNIT_Z,
		Wml::Vector3f::UNIT_Y,
		m_pCamera->GetTarget() );
	NotifyViewChanged();
}

void CViewController::ViewRight()
{
	float fViewDist = ( m_pCamera->GetTarget() - m_pCamera->GetLocation() ).Length();
	m_pCamera->SetTargetFrame(
		m_pCamera->GetTarget() + fViewDist * Wml::Vector3f::UNIT_X,
		Wml::Vector3f::UNIT_Z,
		Wml::Vector3f::UNIT_Y,
		m_pCamera->GetTarget() );
	NotifyViewChanged();
}

void CViewController::ViewTop()
{
	float fViewDist = ( m_pCamera->GetTarget() - m_pCamera->GetLocation() ).Length();
	m_pCamera->SetTargetFrame(
		m_pCamera->GetTarget() + fViewDist * Wml::Vector3f::UNIT_Y,
		-Wml::Vector3f::UNIT_X,
		-Wml::Vector3f::UNIT_Z,
		m_pCamera->GetTarget() );
	NotifyViewChanged();
}

void CViewController::ViewBottom()
{
	float fViewDist = ( m_pCamera->GetTarget() - m_pCamera->GetLocation() ).Length();
	m_pCamera->SetTargetFrame(
		-(m_pCamera->GetTarget() + fViewDist * Wml::Vector3f::UNIT_Y),
		Wml::Vector3f::UNIT_X,
		Wml::Vector3f::UNIT_Z,
		m_pCamera->GetTarget() );
	NotifyViewChanged();
}

void CViewController::ViewFront()
{
	float fViewDist = ( m_pCamera->GetTarget() - m_pCamera->GetLocation() ).Length();
	m_pCamera->SetTargetFrame(
		m_pCamera->GetTarget() + fViewDist * Wml::Vector3f::UNIT_Z,
		-Wml::Vector3f::UNIT_X,
		Wml::Vector3f::UNIT_Y,
		m_pCamera->GetTarget() );
	NotifyViewChanged();
}

void CViewController::ViewBack()
{
	float fViewDist = ( m_pCamera->GetTarget() - m_pCamera->GetLocation() ).Length();
	m_pCamera->SetTargetFrame(
		m_pCamera->GetTarget() + -fViewDist * Wml::Vector3f::UNIT_Z,
		Wml::Vector3f::UNIT_X,
		Wml::Vector3f::UNIT_Y,
		m_pCamera->GetTarget() );
	NotifyViewChanged();
}

void CViewController::ViewThreeQuarter()
{
	// [RMS] this is borken, but it works for what I need right now...

	float fViewDist = ( m_pCamera->GetTarget() - m_pCamera->GetLocation() ).Length();
	Wml::Frame3f frame;
	Wml::Matrix3f mat1, mat2;
	mat1.FromAxisAngle( Wml::Vector3f(1,0,0), 45.0f * Wml::Mathf::DEG_TO_RAD );
	frame.Rotate( mat1 );
	mat2.FromAxisAngle( Wml::Vector3f(0,1,0), 45.0f * Wml::Mathf::DEG_TO_RAD );
	frame.Rotate( mat2 );
	m_pCamera->SetTargetFrame(
		m_pCamera->GetTarget() + -fViewDist * frame.Z(),
		frame.X(),
		frame.Y(),
		m_pCamera->GetTarget() );
	NotifyViewChanged();
}


void CViewController::NotifyViewChanged()
{
	std::set<IViewListener *>::iterator cur(m_vListeners.begin()), end(m_vListeners.end());
	while ( cur != end )
		(*cur++)->OnViewChanged();
}


Wml::SerializerRecord * CViewController::Serialize( Wml::Serializer * pSerializer )
{
	// return camera record
	return m_pCamera->Serialize(pSerializer);
}

bool CViewController::Serialize( Wml::SerializerRecord * pRecord, Wml::Serializer * pSerializer )
{
	// restore camera
	m_pCamera->Serialize(pRecord, pSerializer);

	// update camera
	UpdateCamera();

	return true;
}