// Ryan Schmidt - rms@unknownroad.com
// Copyright (c) 2005-2010
// Distributed under the Boost Software License, Version 1.0.
// http://www.boost.org/LICENSE_1_0.txt


#ifndef _RMSUI_HUD_ITEM_H_
#define _RMSUI_HUD_ITEM_H_

#include <string>
#include <set>

#include <WmlAxisAlignedBox2.h>
#include <WmlVector2.h>

#include "GestureStrokes.h"

namespace rmsui
{

class HUDEventHandler
{
public:
	virtual ~HUDEventHandler() {}

protected:
	HUDEventHandler() {}
};

class HUDItem
{
public:

	enum DockMode {
		NoDocking,
		DockLeft,
		DockRight,
		DockBottom,
		DockTop
	};

	HUDItem(const std::string & name) { m_name = name; m_bVisible = true; }
	virtual ~HUDItem() {}

	/*
	 * attributes
	 */

	std::string & Name() { return m_name; }
	const std::string & Name() const { return m_name; }

	Wml::AxisAlignedBox2f & Bounds() { return m_bounds; }
	const Wml::AxisAlignedBox2f & Bounds() const { return m_bounds; }	

	bool & Visible() { return m_bVisible; }
	const bool & Visible() const { return m_bVisible; }

	void SetDockMode( DockMode eMode ) { m_eDockMode = eMode; }
	DockMode GetDockMode() { return m_eDockMode; }

	/*
	 * overrides
	 */
	virtual void Render() = 0;

	virtual HUDItem * BeginConsume( const Wml::Vector2f & vPoint ) { return NULL; }
	virtual HUDItem * Consume( const Wml::Vector2f & vNextPoint ) { return NULL; }
	virtual void EndConsume() { }

	virtual HUDItem * TestLineStroke( const GestureLineStroke & stroke ) { return NULL; }

	virtual void Translate( float fX, float fY, bool bRelative );

	/*
	 * event handling
	 */

	void AddListener( HUDEventHandler * pHandler ) { m_vHandlers.insert(pHandler); }
	void RemoveListener( HUDEventHandler * pHandler ) { m_vHandlers.erase(pHandler); }

protected:
	std::string m_name;
	std::set<HUDEventHandler *> m_vHandlers;
	bool m_bVisible;

	Wml::AxisAlignedBox2f m_bounds;

	DockMode m_eDockMode;
};




}   // namespace rmsui



#endif // _RMSUI_HUD_ITEM_H_