// Ryan Schmidt - rms@unknownroad.com
// Copyright (c) 2005-2010
// Distributed under the Boost Software License, Version 1.0.
// http://www.boost.org/LICENSE_1_0.txt


#ifndef _RMSUI_HUD_TOOLBAR_H_
#define _RMSUI_HUD_TOOLBAR_H_

#include "HUDItem.h"
#include "HUDButton.h"
#include <vector>

namespace rmsui
{

class IHUDToolbarEventHandler : public HUDEventHandler
{
public:
	virtual ~IHUDToolbarEventHandler() {}

	virtual void OnHUDToolbarMove() {}
	virtual void OnHUDToolbarHide() {}
};


class HUDToolbar : public HUDItem
{
public:

	enum Orientation {
		Horizontal, Vertical
	};

	HUDToolbar(const std::string & name, bool bAllowClose, Orientation eOrientation = Horizontal );
	virtual ~HUDToolbar();

	/*
	 * overrides
	 */
	virtual HUDItem * BeginConsume( const Wml::Vector2f & vPoint );
	virtual HUDItem * Consume( const Wml::Vector2f & vNextPoint );
	virtual void EndConsume();

	virtual HUDItem * TestLineStroke( const GestureLineStroke & stroke );

	virtual void Render();

	virtual void Translate( float fX, float fY, bool bRelative );

	/*
	 * button control
	 */
	void AppendItem( HUDItem * pItem );
	void ClearItems();

	// [RMS: remove]
	void AppendButton( HUDButton * pButton ) { AppendItem(pButton); }
	void ClearButtons() { ClearItems(); }
	

	/*
	 * toolbar options
	 */
	Orientation GetOrientation() { return m_eOrientation; }
	void SetOrientation( Orientation eOrientation );

	float GetMinButtonSize() { return m_fMinButtonSize; }
	void SetMinButtonSize( float fMinButtonSize ) { m_fMinButtonSize = fMinButtonSize; RegenerateLayout(); }

protected:

	HUDButton * m_pCloseButton;
	bool m_bAllowClose;

	std::vector< HUDItem * > m_vItems;

	float m_fButtonSpacing;

	Orientation m_eOrientation;

	float m_fMinButtonSize;

	void InsertItem_Horizontal( HUDItem * pItem );
	void InsertItem_Vertical( HUDItem * pItem );

	void NotifyToolbarMove();
	void NotifyToolbarHide();

	// layout controls
	void InitializeToolbar();

	// compute bounds of toolbar
	void ComputeBounds();

	// re-set toolbar if options change
	void RegenerateLayout();


	// internal class to handle events for close button
	class CloseButtonEventHandler : public rmsui::HUDButtonEventHandler {
	public:
		CloseButtonEventHandler( HUDToolbar * pToolbar ) { m_pToolbar = pToolbar; }
		virtual void OnDragDelta( const Wml::Vector2f & vDelta );
		virtual void OnCross();
	protected:
		HUDToolbar * m_pToolbar;
	};
	friend class CloseButtonEventHandler;

	CloseButtonEventHandler m_closeButtonHandler;
};


}   // namespace rmsui



#endif // _RMSUI_HUD_ITEM_H_