// Ryan Schmidt - rms@unknownroad.com
// Copyright (c) 2005-2010
// Distributed under the Boost Software License, Version 1.0.
// http://www.boost.org/LICENSE_1_0.txt


#include "StdAfx.h"
#include ".\inputeventstream.h"

#include <limits>


using namespace rmsui;

std::map< CWnd *, CInputEventStream * > CInputEventStream::s_streams;
CurrentEventStreamKey CInputEventStream::s_CurrentKey = 0;


CInputEventStream::CInputEventStream(void)
{
	m_pCaptureTarget = NULL;
}

CInputEventStream::~CInputEventStream(void)
{
}

void CInputEventStream::SetCurrentEventStream( CurrentEventStreamKey key )
{
	s_CurrentKey = key;	
}



CInputEventStream * CInputEventStream::Get()
{
	ASSERT(s_CurrentKey != 0);

	if ( s_streams[s_CurrentKey] == NULL )
		s_streams[s_CurrentKey] = new CInputEventStream();
	return s_streams[s_CurrentKey];
}


void CInputEventStream::SendEvent( const InputEventData & event )
{
	if ( m_pCaptureTarget != NULL ) {

		// send to capture target
		IInputEventListener::EventState state = m_pCaptureTarget->ProcessInputEvent( event );
		if ( state == IInputEventListener::EndCapture )
			m_pCaptureTarget = NULL;

	} else { 

		std::vector<IInputEventListener * >::iterator cur( m_vEventHandlers.begin()), end(m_vEventHandlers.end());

		bool bConsumed = false;
		while ( cur != end && !bConsumed ) {

			IInputEventListener * pListener = (*cur++);

			IInputEventListener::EventState state = pListener->ProcessInputEvent(event);

			if ( state == IInputEventListener::Consumed ) {
				bConsumed = true;
			} else if ( state == IInputEventListener::StartCapture ) {
				bConsumed = true;
				m_pCaptureTarget = pListener;
			} else if ( state == IInputEventListener::EndCapture ) {
				ASSERT(0);		// should not happen!
				bConsumed = true;
				m_pCaptureTarget = NULL;
			}
		}

	}
}

void CInputEventStream::AppendLeftMouseDownEvent( int x, int y )
{
	InputEventData data;
	data.eType = InputEventData::MouseEvent;

	data.nMouseX = x;
	data.nMouseY = y;
	data.LeftButton.bDown = true;
	data.LeftButton.bWasDown = false;

	SendEvent( data );
}

void CInputEventStream::AppendLeftMouseUpEvent( int x, int y )
{
	InputEventData data;
	data.eType = InputEventData::MouseEvent;

	data.nMouseX = x;
	data.nMouseY = y;
	data.LeftButton.bDown = false;
	data.LeftButton.bWasDown = true;

	SendEvent( data );
}

void CInputEventStream::AppendLeftMouseMoveEvent( int x, int y )
{
	InputEventData data;
	data.eType = InputEventData::MouseEvent;

	data.nMouseX = x;
	data.nMouseY = y;
	data.LeftButton.bDown = true;
	data.LeftButton.bWasDown = true;

	SendEvent( data );
}


void CInputEventStream::AppendRightMouseDownEvent( int x, int y )
{
	InputEventData data;
	data.eType = InputEventData::MouseEvent;

	data.nMouseX = x;
	data.nMouseY = y;
	data.RightButton.bDown = true;
	data.RightButton.bWasDown = false;

	SendEvent( data );
}

void CInputEventStream::AppendRightMouseUpEvent( int x, int y )
{
	InputEventData data;
	data.eType = InputEventData::MouseEvent;

	data.nMouseX = x;
	data.nMouseY = y;
	data.RightButton.bDown = false;
	data.RightButton.bWasDown = true;

	SendEvent( data );
}

void CInputEventStream::AppendRightMouseMoveEvent( int x, int y )
{
	InputEventData data;
	data.eType = InputEventData::MouseEvent;

	data.nMouseX = x;
	data.nMouseY = y;
	data.RightButton.bDown = true;
	data.RightButton.bWasDown = true;

	SendEvent( data );
}
