// Ryan Schmidt - rms@unknownroad.com
// Copyright (c) 2005-2010
// Distributed under the Boost Software License, Version 1.0.
// http://www.boost.org/LICENSE_1_0.txt


#pragma once

#include "SceneObject.h"
#include "ToolManager.h"
#include <set>
#include <vector>

namespace rmsui {

class SceneObjectManager
{
public:
	SceneObjectManager(void);
	~SceneObjectManager(void);

	virtual bool TestForObjectHit( const Wml::Ray3f & ray );
	virtual bool SelectObject( const Wml::Ray3f & ray, bool bAddToCurrent = false );
	virtual bool DeSelectObject( const Wml::Ray3f & ray );
	virtual bool SelectObject( SceneObject * pObject );

	virtual void SelectAll();
	virtual void DeSelectAll();

	virtual void Render();
	
	virtual void AddObject( SceneObject * pObject, bool bSelected = false );
	virtual void RemoveObject( SceneObject * pObject );
	virtual void RemoveObjectAndChildren( SceneObject * pObject );

	// [RMS] this is kind of hacky...can we do this w/o SceneObjectManager knowing about Tool3DManager ??
	virtual void UpdateToolManager( Tool3DManager * pToolManager );

	//! returns number of scene objects in existence
	virtual unsigned int GetSceneObjectCount() { return (unsigned int)m_entities.size(); }

	//! returns false if vector is not large enough for all objects  (will not resize vector, and starts storing at 0)
	virtual bool GetSceneObjects( std::vector<SceneObject *> & vObjects );

	//! get list of selected entities
	virtual const std::vector<SceneObject *> & GetSelectedEntities();

protected:

	struct SceneEntity {
		SceneObject * m_pObject;
		bool m_bSelected;

		bool m_bToolChildSelected;
		SceneObject * m_pToolChildSelection;

		inline bool operator<( const SceneEntity & entity2 ) const;
	};

	typedef std::set<SceneEntity> EntityList;

	EntityList m_entities;

	
	EntityList m_selectedEntities;
	std::vector<SceneObject *> m_vecSelectedEntities;

	bool AddSelectedEntity( SceneEntity * pEntity );
	bool RemoveSelectedEntity( SceneEntity * pEntity );
	void ClearSelectedEntities();

	SceneEntity * FindClosestHitEntity( const Wml::Ray3f & ray, SceneObject ** pToolChildSelection );
};


inline bool SceneObjectManager::SceneEntity::operator <( const rmsui::SceneObjectManager::SceneEntity & entity2 ) const
{
	return m_pObject < entity2.m_pObject;
}

inline const std::vector<SceneObject *> & SceneObjectManager::GetSelectedEntities()
{
	return m_vecSelectedEntities;
}


} // end namespace rmsui