// Ryan Schmidt - rms@unknownroad.com
// Copyright (c) 2004-2010
// Distributed under the Boost Software License, Version 1.0.
// http://www.boost.org/LICENSE_1_0.txt


#ifndef RMS_OPENGL_WML_COLORS_H
#define RMS_OPENGL_WML_COLORS_H

#include "WmlExt.h"

#include <WmlColorRGB.h>
#include <WmlVector4.h>

namespace Wml
{

class WMLEXT_ITEM ColorRGBA : public Vector4f
{
public:
	ColorRGBA( float r, float g, float b )
		: Vector4f(r,g,b, 1.0f) {}
	ColorRGBA( float r, float g, float b, float a )
		: Vector4f(r,g,b,a) {}
	ColorRGBA( ColorRGB & col ) 
		: Vector4f( col.r, col.g, col.b, 1.0f) {}
	ColorRGBA( unsigned char r, unsigned char g, unsigned char b, unsigned char a )
		: Vector4f( (float)r / 255.0f, (float)g / 255.0f, (float)b / 255.0f, (float)a / 255.0f ) {}

    // member access
	float R () const { return X(); }
	float& R () { return X(); }
    float G () const { return Y(); }
    float& G () { return Y(); }
    float B () const { return Z(); }
    float& B () { return Z(); }
    float A () const { return W(); }
    float& A () { return W(); }


    static const ColorRGBA BLACK; // = (0,0,0) 
    static const ColorRGBA WHITE; // = (1,1,1)
    static const ColorRGBA RED; // = (1,0,0) 
    static const ColorRGBA GREEN; // = (0,1,0)
    static const ColorRGBA BLUE; // = (0,0,1)
	static const ColorRGBA YELLOW; // = (1,1,0)
	static const ColorRGBA MAGENTA; // = {1,0,1}

	static unsigned char ToByte( float fValue );
};


class WMLEXT_ITEM ColorConstants
{
public:	

    static const ColorRGB BLACK; // = (0,0,0) 
    static const ColorRGB WHITE; // = (1,1,1)
    static const ColorRGB RED; // = (1,0,0) 
    static const ColorRGB GREEN; // = (0,1,0)
    static const ColorRGB BLUE; // = (0,0,1)
	static const ColorRGB YELLOW; // = (1,1,0)
	static const ColorRGB MAGENTA; // = {1,0,1}

private:
	ColorConstants() {}
};

} // namespace Wml

#endif RMS_OPENGL_WML_COLORS_H 
