// Ryan Schmidt - rms@unknownroad.com
// Copyright (c) 2004-2010
// Distributed under the Boost Software License, Version 1.0.
// http://www.boost.org/LICENSE_1_0.txt


#include "StdAfx.h"
#include "WmlExtDistVec2Box2.h"

using namespace Wml;

template <class Real>
Real Wml::SqrDistance( const Vector2<Real>& rkPoint, const Box2<Real>& rkBox )
{
    // compute coordinates of point in box coordinate system
    Vector2<Real> kDiff = rkPoint - rkBox.Center();
    Vector2<Real> kClosest( kDiff.Dot(rkBox.Axis(0)), kDiff.Dot(rkBox.Axis(1)) );

    // project test point onto box
    Real fSqrDistance = (Real)0.0;
    Real fDelta;

    if ( kClosest.X() < -rkBox.Extent(0) )
    {
        fDelta = kClosest.X() + rkBox.Extent(0);
        fSqrDistance += fDelta*fDelta;
        kClosest.X() = -rkBox.Extent(0);
    }
    else if ( kClosest.X() > rkBox.Extent(0) )
    {
        fDelta = kClosest.X() - rkBox.Extent(0);
        fSqrDistance += fDelta*fDelta;
        kClosest.X() = rkBox.Extent(0);
    }

    if ( kClosest.Y() < -rkBox.Extent(1) )
    {
        fDelta = kClosest.Y() + rkBox.Extent(1);
        fSqrDistance += fDelta*fDelta;
        kClosest.Y() = -rkBox.Extent(1);
    }
    else if ( kClosest.Y() > rkBox.Extent(1) )
    {
        fDelta = kClosest.Y() - rkBox.Extent(1);
        fSqrDistance += fDelta*fDelta;
        kClosest.Y() = rkBox.Extent(1);
    }

    return fSqrDistance;
}


template <class Real>
Real Wml::Distance( const Vector2<Real>& rkPoint, const Box2<Real>& rkBox )
{
    return Math<Real>::Sqrt(SqrDistance(rkPoint,rkBox));
}





namespace Wml
{
template WMLEXT_ITEM float SqrDistance<float> (const Vector2<float>& rkPoint, const Box2<float>& rkBox);
template WMLEXT_ITEM float Distance<float> (const Vector2<float>& rkPoint, const Box2<float>& rkBox);

template WMLEXT_ITEM double SqrDistance<double> (const Vector2<double>& rkPoint, const Box2<double>& rkBox);
template WMLEXT_ITEM double Distance<double> (const Vector2<double>& rkPoint, const Box2<double>& rkBox);
}
