// Ryan Schmidt - rms@unknownroad.com
// Copyright (c) 2004-2010
// Distributed under the Boost Software License, Version 1.0.
// http://www.boost.org/LICENSE_1_0.txt


#ifndef __RMS_WML_EXT_VECTOR_UTIL_H__
#define __RMS_WML_EXT_VECTOR_UTIL_H__

#include "WmlExt.h"

#include <WmlVector2.h>
#include <WmlVector3.h>
#include <WmlMatrix3.h>
#include <WmlAxisAlignedBox2.h>
#include <WmlAxisAlignedBox3.h>
#include <WmlBox3.h>

namespace Wml
{

	/*
	 * if bToZAxis is false, compute matrix that rotates Z axis into vAlignWith
	 * if bToZAxis is true, compute matrix that rotates vAlignWith into Z axis
	 */
	template <class Real>
	void WMLEXT_ITEM ComputeAlignZAxisMatrix( const Vector3<Real> & vAlignWith,
											  Matrix3<Real> & matrix, bool bToZAxis = false );

	template <class Real>
	void WMLEXT_ITEM ComputeAlignAxisMatrix( const Vector3<Real> & vInitial,
											 const Vector3<Real> & vAlignWith, Matrix3<Real> & matrix );

	//! compute vectors in a plane perpendicular to vIn
	template <class Real>
	void WMLEXT_ITEM ComputePerpVectors( const Vector3<Real> & vIn,
										 Vector3<Real> & vOut1, Vector3<Real> & vOut2,
										 bool bInIsNormalized = false);

	template <class Real>
	void WMLEXT_ITEM ToGLMatrix( const Matrix3<Real> & matrix, Real glMatrix[16] );

	void WMLEXT_ITEM Union( Wml::AxisAlignedBox3f & dest, const Wml::AxisAlignedBox3f & with );
	void WMLEXT_ITEM Union( Wml::AxisAlignedBox3f & dest, const Wml::Vector3f & point );

	void WMLEXT_ITEM Union( Wml::AxisAlignedBox2f & dest, const Wml::AxisAlignedBox2f & with );

	float WMLEXT_ITEM Volume( const Wml::AxisAlignedBox3f & box );

	//! create box that contains sphere
	template <class Real>
	void WMLEXT_ITEM FitAABox( const Wml::Vector3<Real> & vCenter, Real fRadius, Wml::AxisAlignedBox3<Real> & aaBox );

	template <class Real>
	void WMLEXT_ITEM ConvertBox( const Wml::AxisAlignedBox3<Real> & fromAABox, Wml::Box3<Real> & toOBox );

	template <class Real>
	void WMLEXT_ITEM ScaleBox( Wml::Box3<Real> & toOBox, Real fScaleX, Real fScaleY, Real fScaleZ );


	bool WMLEXT_ITEM Contained( const Wml::AxisAlignedBox3f & box, float fX, float fY, float fZ );
	bool WMLEXT_ITEM Contained( const Wml::AxisAlignedBox2f & box, float fX, float fY );

	template <class Real>
	Wml::Vector3<Real> WMLEXT_ITEM Center( const Wml::AxisAlignedBox3<Real> & box );
	template <class Real>
	Wml::Vector2<Real> WMLEXT_ITEM Center( const Wml::AxisAlignedBox2<Real> & box );

	template <class Real>
	Real WMLEXT_ITEM GetDimension( const Wml::AxisAlignedBox3<Real> & box, int nDimension );
	template <class Real>
	Real WMLEXT_ITEM GetDimension( const Wml::AxisAlignedBox2<Real> & box, int nDimension );

	template <class Real>
	Real WMLEXT_ITEM MaxDimension( const Wml::AxisAlignedBox3<Real> & box );

	template<class Real>
	void WMLEXT_ITEM Translate( Wml::AxisAlignedBox2<Real> & box, Real fX, Real fY, bool bRelative );

	// casting operators for vector
	Wml::Vector3f WMLEXT_ITEM VectorCastdf( const Wml::Vector3d & vec );
	Wml::Vector3d WMLEXT_ITEM VectorCastfd( const Wml::Vector3f & vec );

	template<class Real>
	Real WMLEXT_ITEM Clamp( const Real & fValue, const Real & fMin, const Real & fMax );


}  // namespace Wml



#endif // __RMS_WML_EXT_VECTOR_UTIL_H__