// Ryan Schmidt - rms@unknownroad.com
// Copyright (c) 2004-2010
// Distributed under the Boost Software License, Version 1.0.
// http://www.boost.org/LICENSE_1_0.txt


#include "StdAfx.h"
#include "WmlInterpolatingSpline2.h"

using namespace Wml;

template<class Real>
InterpolatingSpline2<Real>::InterpolatingSpline2()

{
}

template<class Real>
InterpolatingSpline2<Real>::InterpolatingSpline2(const std::vector<Wml::Vector2<Real> > & vControlPoints)
{
	m_vControlPoints = vControlPoints;
}

template<class Real>
Wml::Vector2<Real> InterpolatingSpline2<Real>::GetPosition( Real fT )
{
	size_t nCount = m_vControlPoints.size();
	if ( nCount < 4 )
		return Wml::Vector2<Real>::ZERO;
	Real fStepSize = 1.0f / (Real)(nCount-3);

	int nStep = (int)(fT / fStepSize);
	Real fUseT = fT - (Real)nStep * fStepSize;

	Wml::Vector2<Real> & vP0 = m_vControlPoints[ nStep ];
	Wml::Vector2<Real> & vP1 = m_vControlPoints[ nStep+1 ];
	Wml::Vector2<Real> & vP2 = m_vControlPoints[ nStep+2 ];
	Wml::Vector2<Real> & vP3 = m_vControlPoints[ nStep+3 ];

	Wml::Vector2<Real> vPoint( (Real)2.0 * vP1 );
	vPoint += (vP2 - vP0) * fUseT;
	vPoint += ((Real)2*vP0 - (Real)5*vP1 + (Real)4*vP2 - vP3) * fUseT * fUseT;
	vPoint += (vP3 - vP0 + (Real)3*vP1 - (Real)3*vP2) *  fUseT * fUseT * fUseT;
	vPoint *= (Real)0.5;

	return vPoint;
}



//----------------------------------------------------------------------------
// explicit instantiation
//----------------------------------------------------------------------------
namespace Wml
{
template WMLEXT_ITEM class InterpolatingSpline2<float>;
template WMLEXT_ITEM class InterpolatingSpline2<double>;
}
//----------------------------------------------------------------------------
