// Ryan Schmidt - rms@unknownroad.com
// Copyright (c) 2004-2010
// Distributed under the Boost Software License, Version 1.0.
// http://www.boost.org/LICENSE_1_0.txt


#include "StdAfx.h"

#include "WmlPGMImage.h"
#include <fstream>

using namespace Wml;

PGMImage::PGMImage()
{
	Resize(0,0);
}

PGMImage::PGMImage(unsigned int nWidth, unsigned int nHeight)
{
	Resize(nWidth, nHeight);
}

unsigned char PGMImage::GetPixelByte( unsigned int x, unsigned int y )
{
	return m_vValues[y*m_nWidth + x];
}


float PGMImage::GetPixelFloat( unsigned int x, unsigned int y )
{
	unsigned char nPixel = m_vValues[y * m_nWidth + x];
	float fValue = (float)nPixel / 255.0f;
	return ( fValue > 1.0f ) ? 1.0f : fValue;
}

void PGMImage::SetPixel( float fColor, unsigned int x, unsigned int y )
{
	unsigned char nColor = (unsigned char)(fColor * 255.0f);
	SetPixel( nColor > 255 ? (unsigned char)255 : nColor, x, y );
}


void PGMImage::SetPixel( unsigned char nColor, unsigned int x, unsigned int y )
{
	m_vValues[y * m_nWidth + x] = nColor;
}


void PGMImage::Resize( unsigned int nWidth, unsigned int nHeight )
{
	m_nWidth = nWidth;
	m_nHeight = nHeight;
	m_vValues.resize( m_nWidth * m_nHeight );
}


bool PGMImage::Load( const char * pFilename )
{
	char nFormat[10];
	unsigned int nResolutionX, nResolutionY, nMaxValue;
	bool bAscii = false;

	std::ifstream in(pFilename);
	if (!in)
		goto fail;

	in >> nFormat;
	in >> nResolutionX >> nResolutionY >> nMaxValue;
	if (nMaxValue != 255)
		goto fail;

	if ( strcmp(nFormat, "P2") == 0 )
		bAscii = true;
	else if ( strcmp(nFormat, "P5") != 0 )
		goto fail;

	Resize( nResolutionX, nResolutionY );

	// read comments ?? (how) ??

	// read image
	for (unsigned int i = 0; i < nResolutionX * nResolutionY; ++i) {
		if (bAscii) {
			unsigned int nValue;
			in >> nValue;
			m_vValues[i] = nValue;
		} else {
			unsigned char m;
			in.read((char *)&m, 1);
			m_vValues[i] = m;
		}
		if (! in )
			goto fail;
	}

	// done with file
	in.close();
	return true;

fail:
	in.close();
	return false;
}

bool PGMImage::Save( const char * pFilename )
{
	std::ofstream outFile(pFilename);
	if (!outFile)
		return false;

	outFile << "P2" << std::endl;
	outFile << m_nWidth << " " << m_nHeight << std::endl;
	outFile << "255" << std::endl;

	for ( unsigned int y = 0; y < m_nHeight; ++y ) {
		for ( unsigned int x = 0; x < m_nWidth; ++x ) {
			unsigned int nValue = GetPixelByte(x,y);
			outFile << nValue << " ";
		}
		outFile << std::endl;
	}

	if ( !outFile ) {
		outFile.close();
		return false;
	} else {
		outFile.close();
		return true;
	}
}
