// Ryan Schmidt - rms@unknownroad.com
// Copyright (c) 2004-2010
// Distributed under the Boost Software License, Version 1.0.
// http://www.boost.org/LICENSE_1_0.txt


#include "StdAfx.h"

#include "WmlSerializer.h"

using namespace Wml;


#include <fstream>


Serializer::Serializer()
{

}


Serializer::~Serializer()
{

}


SerializerRecord * Serializer::CreateRecord( SerializerRecord::RecordType nType )
{
	SerializerRecord * pRecord = new SerializerRecord(nType);

	// save in list
	m_recordList.push_back( pRecord );

	// save in map
	m_recordMap[ pRecord->GetRecordID() ] = pRecord;

	return pRecord;
}

SerializerRecord * Serializer::CreateRecord( SerializerRecord::RecordID nID, SerializerRecord::RecordType nType )
{
	SerializerRecord * pRecord = new SerializerRecord(nID, nType);

	// save in list
	m_recordList.push_back( pRecord );

	// save in map
	m_recordMap[ pRecord->GetRecordID() ] = pRecord;

	return pRecord;
}




void Serializer::RestoreRecords()
{
	size_t nReaders = m_vReaders.size();
	size_t nCount = m_recordList.size();
	for ( unsigned int i = 0; i < nCount; ++i ) {

		SerializerRecord * pRecord = m_recordList[i];

		bool bRead = false;
		for ( unsigned int j = 0; j < nReaders && !bRead; ++j ) {

			if ( m_vReaders[j]->ReadRecord(pRecord) != false )
				bRead = true;
		}
	}
	
}




SerializerRecord * Serializer::FindRecord( SerializerRecord::RecordID nID )
{
	if ( m_recordMap.find( nID ) != m_recordMap.end() )
		return m_recordMap[nID];
	return NULL;
}


void Serializer::AddItem( SerializerRecord::RecordID nID, void * pItem )
{
	m_itemMap[nID] = pItem;
}

void * Serializer::FindItem( SerializerRecord::RecordID nID )
{
	if ( m_itemMap.find( nID ) != m_itemMap.end() )
		return m_itemMap[nID];
	return NULL;
}




bool Serializer::Write( const char * pFilename )
{
	std::ofstream out( pFilename );
	if (!out)
		return false;

	
	// WRITE header
	out << "ShapeShop Scene File v0.0.1" << std::endl << std::endl;

	// WRITE number of records
	out << (unsigned int)m_recordList.size() << std::endl << std::endl;

	std::vector<unsigned char> buf;
	buf.reserve(1024);

	size_t nCount = m_recordList.size();
	for ( unsigned int i = 0; i < nCount; ++i ) {

		SerializerRecord * pRecord = m_recordList[i];
		SerializerRecord::RecordID nID = pRecord->GetRecordID();
		
		// WRITE record ID
		out << nID << std::endl;

		buf.resize(0);
		bool bResult = pRecord->GetBytes( buf );
		if ( !bResult ) {
			buf.resize(0);
			SerializerRecord::GetEmptyRecordBytes( buf );
		}

		// WRITE record size
		out << (unsigned int)buf.size() << std::endl;

		// WRITE record data
		size_t nCount = buf.size();
		for ( unsigned int j = 0; j < nCount; ++j )
			out << (int)buf[j] << " ";
		out << std::endl;


		// WRITE newlinne
		out << std::endl;
	}

	out.close();

	return true;

}

bool Serializer::Read( const char * pFilename )
{
	std::ifstream in( pFilename );
	if (!in)
		return false;

	
	// READ header
	char header[256];
	in.getline(header, 256 );
	if ( strcmp(header, "ShapeShop Scene File v0.0.1") != 0 )
		return false;


	// READ number of records
	unsigned int nRecordCount;
	in >> nRecordCount;

	std::vector<unsigned char> buf;
	buf.reserve(1024);

	int tmp;
	for ( unsigned int i = 0; i < nRecordCount; ++i ) {

		
		// READ record ID
		SerializerRecord::RecordID nID;
		in >> nID;

		// READ record size
		unsigned int nRecordSize;
		in >> nRecordSize;

		// READ record data
		buf.resize(nRecordSize);
		for ( unsigned int j = 0; j < nRecordSize; ++j ) {
			in >> tmp;
			buf[j] = (unsigned char)tmp;
		}

		if (!in)
			return false;

		// CREATE record
		SerializerRecord * pRecord = new SerializerRecord(0,0);
		if ( ! pRecord->SetBytes( buf ) ) {
			delete pRecord;
			return false;
		}

		m_recordList.push_back( pRecord );
		m_recordMap[ nID ] = pRecord;
	}

	if (!in)
		return false;

	in.close();

	return true;
}
