// Author: Ryan Schmidt  (rms@unknownroad.com)  http://www.unknownroad.com
// Copyright (c) 2006. All Rights Reserved.
// The SketchFramework Library is supplied "AS IS". The Author disclaims all warranties, expressed or implied, including, 
// without limitation, the warranties of merchantability and of fitness for any purpose. The Author assume no liability for 
// direct, indirect, incidental, special, exemplary, or consequential damages, which may result from the use of the 
// SketchFramework Library, even if advised of the possibility of such damage. Permission is hereby granted to use, copy, 
// modify, and distribute this source code, or portions hereof, for any purpose, without fee, subject to the following restrictions:
// 1) The origin of this source code must not be misrepresented.
// 2) This Copyright notice may not be removed or altered from any source or altered source distribution.
// The Author specifically permits, without fee, the use of this source code as a component in commercial products.

#ifndef __RMS_DEBUG_H__
#define __RMS_DEBUG_H__

#ifdef _WIN32
#include "windows.h"
#pragma warning(disable:4505)
#endif

#include <string>
using namespace std;

#include <stdio.h>
#include <stdarg.h>

#ifndef ASSERT
#define ASSERT(x)  { if(! (x) ) abort(); }
#endif

static void _RMSInfo(char * str, ...)
{	
	static char buf[1024];
	va_list args;

	va_start(args, str);
	vsprintf(buf, str, args);
#ifdef _WIN32
	OutputDebugString(buf);
#else
	fprintf(stderr, "%s", buf);
#endif
	va_end(args);
}

static string _RMSInfoString(char * str, ...)
{	
	static char buf[1024];
	va_list args;

	va_start(args, str);
	vsprintf(buf, str, args);
	va_end(args);
	return string(buf);
}


/*
 *
 * OpenGL debug utility functions
 *
 */


#ifdef __RMS_DEBUG_WANT_GL


#include <GL/gl.h>


static char _RMSDEBUG_GLErrorStrings[][32] = {  "GL_NO_ERROR",
												"GL_INVALID_ENUM", 
												"GL_INVALID_VALUE", 
												"GL_INVALID_OPERATION", 
												"GL_STACK_OVERFLOW",
												"GL_STACK_UNDERFLOW",
												"GL_OUT_OF_MEMORY",
												"Unknown GL Error"};

static char * glErrorString(GLenum error) 
{
	switch(error) {
		case GL_NO_ERROR: return _RMSDEBUG_GLErrorStrings[0];
		case GL_INVALID_ENUM: return _RMSDEBUG_GLErrorStrings[1];
		case GL_INVALID_VALUE: return _RMSDEBUG_GLErrorStrings[2];
		case GL_INVALID_OPERATION: return _RMSDEBUG_GLErrorStrings[3];
		case GL_STACK_OVERFLOW: return _RMSDEBUG_GLErrorStrings[4];
		case GL_STACK_UNDERFLOW: return _RMSDEBUG_GLErrorStrings[5];
		case GL_OUT_OF_MEMORY: return _RMSDEBUG_GLErrorStrings[6];
		default: return _RMSDEBUG_GLErrorStrings[7];
	}
}

static GLenum _RMSGLError = GL_NO_ERROR;

#define _RMSHaveGLError() (_RMSGLError != GL_NO_ERROR)

static bool glError()
{
	_RMSGLError = glGetError();
	int nextErr = _RMSGLError;
	while (nextErr != GL_NO_ERROR)
		nextErr = glGetError();
	return _RMSGLError != GL_NO_ERROR;
}



#endif // __RMS_DEBUG_WANT_GL

#endif   // __RMS_DEBUG_H__